@@ -784,3 +784,60 @@ class UnionsOfPrimitveTypes:
             ouis: Optional[int | str]
             uisn: int | str | None = None
             uis_with_default: int | str = 123
+
+
+if sys.version_info >= (3, 9):
+
+    @attr.s(auto_attribs=True)
+    class SupportPEP585:
+        """
+        PEP 585 – Type Hinting Generics In Standard Collections
+        https://peps.python.org/pep-0585/
+
+        This means lower-case dict/list/tuple annotations
+        can be used instad of uppercase Dict/List/Tuple.
+        """
+
+        dict_: dict[int, str] = {123: "abc"}
+        list_: list[int] = [123]
+        tuple_: tuple[int] = (123,)
+        dict_no_subscript: dict = {123: "abc"}
+        list_no_subscript: list = [123]
+        tuple_no_subscript: tuple = (123,)
+
+
+@attr.s(auto_attribs=True)
+class HasForwardRef:
+    @attr.s(auto_attribs=True)
+    class CA:
+        x: int = 3
+
+    @attr.s(auto_attribs=True)
+    class CB:
+        sub: "HasForwardRef.CA"
+
+    a: CA
+    b: CB
+
+
+@attr.s(auto_attribs=True)
+class HasBadAnnotation1:
+    data: object
+
+
+@attr.s(auto_attribs=True)
+class HasBadAnnotation2:
+    data: object()  # type: ignore
+
+
+@attr.s(auto_attribs=True)
+class HasIgnoreMetadataRequired:
+    ignore: int = attr.field(metadata={"omegaconf_ignore": True})
+    no_ignore: int = attr.field(metadata={"omegaconf_ignore": False})
+
+
+@attr.s(auto_attribs=True)
+class HasIgnoreMetadataWithDefault:
+
+    ignore: int = attr.field(default=1, metadata={"omegaconf_ignore": True})
+    no_ignore: int = attr.field(default=2, metadata={"omegaconf_ignore": False})
