@@ -5,7 +5,8 @@ import re
 import sys
 from importlib import import_module
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Union
+from types import LambdaType
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from _pytest.python_api import RaisesContext
 from pytest import fixture, mark, param, raises
@@ -29,10 +30,17 @@ from tests import Color, Enum1, User, warns_dict_subclass_deprecated
 
 @fixture(
     params=[
-        "tests.structured_conf.data.dataclasses",
-        "tests.structured_conf.data.attr_classes",
+        param("tests.structured_conf.data.dataclasses", id="dataclasses"),
+        param(
+            "tests.structured_conf.data.dataclasses_pre_311",
+            id="dataclasses_pre_311",
+            marks=mark.skipif(
+                sys.version_info >= (3, 11),
+                reason="python >= 3.11 does not support mutable default dataclass arguments",
+            ),
+        ),
+        param("tests.structured_conf.data.attr_classes", id="attr_classes"),
     ],
-    ids=["dataclasses", "attr_classes"],
 )
 def module(request: Any) -> Any:
     return import_module(request.param)
@@ -981,6 +989,80 @@ class TestConfigs:
         assert cfg.list == [1, 2]
         assert cfg.opt_list is None
 
+    def test_has_bad_annotation1(self, module: Any) -> None:
+        with raises(ValidationError, match="Unexpected type annotation: object"):
+            OmegaConf.structured(module.HasBadAnnotation1)
+
+    def test_has_bad_annotation2(self, module: Any) -> None:
+        with raises(
+            ValidationError,
+            match="Unexpected type annotation: <object object at 0x[a-fA-F0-9]*>",
+        ):
+            OmegaConf.structured(module.HasBadAnnotation2)
+
+    @mark.parametrize(
+        "input_, expected, expected_type, expected_ref_type, expected_object_type",
+        [
+            param(
+                lambda module: module.HasBadAnnotation1,
+                {"data": "???"},
+                AnyNode,
+                Any,
+                None,
+            ),
+            param(
+                lambda module: module.HasBadAnnotation1(123),
+                {"data": 123},
+                AnyNode,
+                Any,
+                None,
+            ),
+            param(
+                lambda module: module.HasBadAnnotation1([1, 2, 3]),
+                {"data": [1, 2, 3]},
+                ListConfig,
+                object,
+                list,
+            ),
+            param(
+                lambda module: module.HasBadAnnotation1({1: 2}),
+                {"data": {1: 2}},
+                DictConfig,
+                object,
+                dict,
+            ),
+            param(
+                lambda module: module.HasBadAnnotation1(module.UserWithDefaultName),
+                {"data": {"name": "bob", "age": "???"}},
+                DictConfig,
+                object,
+                lambda module: module.UserWithDefaultName,
+            ),
+        ],
+    )
+    def test_bad_annotation_allow_objects(
+        self,
+        module: Any,
+        input_: Any,
+        expected: Any,
+        expected_type: Any,
+        expected_ref_type: Any,
+        expected_object_type: Any,
+    ) -> None:
+        """
+        Test how unsupported annotation types are handled when `allow_objects` is True
+        """
+        input_ = input_(module)
+        if isinstance(expected_object_type, LambdaType):
+            expected_object_type = expected_object_type(module)
+
+        cfg = OmegaConf.structured(input_, flags={"allow_objects": True})
+
+        assert cfg == expected
+        assert isinstance(cfg._get_node("data"), expected_type)
+        assert cfg._get_node("data")._metadata.ref_type is expected_ref_type
+        assert cfg._get_node("data")._metadata.object_type is expected_object_type
+
 
 def validate_frozen_impl(conf: DictConfig) -> None:
     with raises(ReadonlyConfigError):
@@ -1010,6 +1092,12 @@ def test_frozen(module: Any) -> None:
     validate_frozen_impl(OmegaConf.structured(FrozenClass()))
 
 
+def test_forward_ref(module: Any) -> None:
+    C = module.HasForwardRef
+    obj = C(a=C.CA(), b=C.CB(C.CA(x=33)))
+    OmegaConf.create(obj)
+
+
 class TestDictSubclass:
     def test_str2str(self, module: Any) -> None:
         with warns_dict_subclass_deprecated(module.DictSubclass.Str2Str):
@@ -1220,6 +1308,14 @@ class TestConfigs2:
 
         assert OmegaConf.merge(cfg, cfg) == cfg
 
+    def test_set_value_after_merge_into_none_dict(self, module: Any) -> None:
+        cfg = OmegaConf.structured(module.DictOptional)
+        merged = OmegaConf.merge(cfg, {"as_none": {"x": 100}})
+        with raises(ValidationError):
+            merged.as_none.x = "abc"
+        with raises(ValidationError):
+            merged.as_none.y = "abc"
+
     @mark.parametrize(
         "update_value,expected",
         [
@@ -1259,6 +1355,22 @@ class TestConfigs2:
         with raises(ValidationError):
             c3.missing.append("xx")
 
+    def test_ignore_metadata_class_with_required_args(self, module: Any) -> None:
+        cfg = OmegaConf.create(module.HasIgnoreMetadataRequired)
+        assert cfg == {"no_ignore": "???"}
+
+    def test_ignore_metadata_instance_with_required_args(self, module: Any) -> None:
+        cfg = OmegaConf.create(module.HasIgnoreMetadataRequired(3, 4))
+        assert cfg == {"no_ignore": 4}
+
+    def test_ignore_metadata_class_with_default_args(self, module: Any) -> None:
+        cfg = OmegaConf.create(module.HasIgnoreMetadataWithDefault)
+        assert cfg == {"no_ignore": 2}
+
+    def test_ignore_metadata_instance_with_default_args(self, module: Any) -> None:
+        cfg = OmegaConf.create(module.HasIgnoreMetadataWithDefault(3, 4))
+        assert cfg == {"no_ignore": 4}
+
 
 class TestStructredConfigInheritance:
     def test_leaf_node_inheritance(self, module: Any) -> None:
@@ -2267,6 +2379,17 @@ class TestUnionsOfPrimitiveTypes:
         assert cfg.uisn is None
         assert cfg.uis_with_default == 123
 
+    @mark.skipif(sys.version_info < (3, 9), reason="requires Python 3.9 or newer")
+    def test_support_pep_585(self, module: Any) -> None:
+        class_ = module.SupportPEP585
+        cfg = OmegaConf.structured(class_)
+        assert _utils.get_type_hint(cfg, "dict_") == Dict[int, str]
+        assert _utils.get_type_hint(cfg, "list_") == List[int]
+        assert _utils.get_type_hint(cfg, "tuple_") == Tuple[int]
+        assert _utils.get_type_hint(cfg, "dict_no_subscript") == Dict[Any, Any]
+        assert _utils.get_type_hint(cfg, "list_no_subscript") == List[Any]
+        assert _utils.get_type_hint(cfg, "tuple_no_subscript") == Tuple[Any, ...]
+
     def test_assign_path_to_string_typed_field(self, module: Any) -> None:
         cfg = OmegaConf.create(module.StringConfig)
         cfg.null_default = Path("hello.txt")
