@@ -89,10 +89,13 @@ PARAMS_SINGLE_ELEMENT_NO_INTERPOLATION: List[Tuple[str, str, Any]] = [
     ("float_no_int", ".1", 0.1),
     ("float_no_decimal", "1.", 1.0),
     ("float_minus", "-.2", -0.2),
-    ("float_underscore", "1.1_1", 1.11),
+    ("float_plus", "+.2", 0.2),
+    ("float_underscore_1", "1.1_1", 1.11),
+    ("float_underscore_2", "1_1_2.1", 112.1),
     ("float_bad_1", "1.+2", "1.+2"),
     ("float_bad_2", r"1\.2", r"1\.2"),
     ("float_bad_3", "1.2_", "1.2_"),
+    ("float_bad_4", "1__1.2", "1__1.2"),
     ("float_exp_1", "-1e2", -100.0),
     ("float_exp_2", "+1E-2", 0.01),
     ("float_exp_3", "1_0e1_0", 10e10),
@@ -100,6 +103,7 @@ PARAMS_SINGLE_ELEMENT_NO_INTERPOLATION: List[Tuple[str, str, Any]] = [
     ("float_exp_5", "1e+03", 1000.0),
     ("float_exp_bad_1", "e-2", "e-2"),
     ("float_exp_bad_2", "01e2", "01e2"),
+    ("float_exp_bad_3", "0_e0", "0_e0"),
     ("float_inf", "inf", math.inf),
     ("float_plus_inf", "+inf", math.inf),
     ("float_minus_inf", "-inf", -math.inf),
@@ -612,6 +616,7 @@ class TestOmegaConfGrammar:
         "x ${ab.cd.ef.gh} y",
         "$ ${foo} ${bar} ${boz} $",
         "${foo:bar}",
+        "${foo-bar:bar-foo}",
         "${foo : bar, baz, boz}",
         "${foo:bar,0,a-b+c*d/$.%@?|}",
         r"\${foo}",
