@@ -46,12 +46,16 @@ from tests import (
     Dataframe,
     Group,
     IllegalType,
+    InnerX,
+    InnerY,
     InterpolationDict,
     InterpolationList,
     MissingDict,
     MissingList,
     OptionalUsers,
     OptTuple,
+    OuterA,
+    OuterB,
     Package,
     Plugin,
     User,
@@ -405,6 +409,16 @@ from tests import (
             [User(name="Bond", age=MISSING)],
             id="list_sc_element_merge_dict",
         ),
+        param(
+            [OuterA(x=InnerX(d=10, p=2)), OuterB(x=InnerY(d=MISSING, p=MISSING, q=10))],
+            OuterB(x=InnerY(d=10, p=2, q=10)),
+            id="nested_configs_merge1",
+        ),
+        param(
+            [OuterA(x=InnerX(d=10, p=2)), OuterB(x=MISSING)],
+            OuterB(x=InnerY(d=10, p=2, q=MISSING)),
+            id="nested_configs_merge2",
+        ),
     ],
 )
 def test_merge(
